==> simple_survey-8.x-1.13.tar.gz: simple_survey/src/Service/BigQueryConnect.php <==
<?php

namespace Drupal\simple_survey\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Datetime\DateFormatter;
use Google\Cloud\BigQuery\BigQueryClient;
use Google\Cloud\Core\ExponentialBackoff;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\simple_survey\Entity\SimpleSurvey;

/**
 * Drupal BigQuery Service.
 */
class BigQueryConnect implements ContainerInjectionInterface {

  /**
   * The module name.
   *
   * @var string
   */
  private $moduleName = 'simple_survey';

  /**
   * Is BigQuery Configured.
   *
   * @var bool
   */
  private $isConfigured = FALSE;

  /**
   * The big query client object.
   *
   * @var \Google\Cloud\BigQuery\BigQueryClient
   */
  private $bigQueryClient;

  /**
   * SimpleSurvey config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $simpleSurveyConfig;

  /**
   * Drupal config service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  private $configFactory;

  /**
   * Drupal logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Drupal messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The current survey.
   *
   * @var \Drupal\simple_survey\Entity\SimpleSurvey
   */
  public $survey;

  /**
   * Service config and DI.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Drupal Messenger service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config factory service.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   Date formatter service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   Drupal logger service.
   */
  public function __construct(
    MessengerInterface $messenger,
    ConfigFactoryInterface $configFactory,
    DateFormatter $dateFormatter,
    LoggerChannelFactoryInterface $loggerFactory
  ) {
    $this->messenger = $messenger;
    $this->configFactory = $configFactory;
    $this->dateFormatter = $dateFormatter;
    $this->logger = $loggerFactory->get($this->moduleName);
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('config.factory'),
      $container->get('date.formatter'),
      $container->get('logger.factory')
    );
  }

  /**
   * Sets the active survey.
   *
   * @param \Drupal\simple_survey\Entity\SimpleSurvey $survey
   *   The active survey.
   */
  public function setSurvey(SimpleSurvey $survey) {
    $this->survey = $survey;

    if ($this->survey->get('bigQueryKeyfile')) {
      $this->bigQueryClient = new BigQueryClient([
        'projectId' => $this->survey->get('bigQueryProjectId'),
        'keyFile' => json_decode($this->survey->get('bigQueryKeyfile'), TRUE),
      ]);

      $this->isConfigured = TRUE;
    }

  }

  /**
   * Checks Simple Survey configured to use BigQuery.
   *
   * @return bool
   *   Returns true if configured, false if not.
   */
  public function checkConfigured() {
    return $this->isConfigured;
  }

  /**
   * Send table data to BigQuery and overwrites existing data.
   *
   * @param string $dataset
   *   The name of the BigQuery dataset.
   * @param string $table
   *   The name of the BigQuery table.
   * @param string $csv
   *   The converted data.
   *
   * @return bool
   *   Returns true if updated, false if failed.
   */
  public function updateTable(string $dataset, string $table, string $csv) {
    $dataset = $this->bigQueryClient->dataset($dataset);
    $table = $dataset->table($table);

    // Loads the CSV output.
    $loadConfig = $table->load($csv);
    // Sets the format to CSV.
    $loadConfig->sourceFormat('CSV');
    // Allows BigQuery to autodetect the header and create columns.
    $loadConfig->autodetect(TRUE);
    // Forces the BigQuery table to be overwritten.
    $loadConfig->writeDisposition('WRITE_TRUNCATE');

    $job = $table->runJob($loadConfig);

    // Poll the job until it is complete.
    $backoff = new ExponentialBackoff(10);
    $backoff->execute(function () use ($job) {
      $job->reload();
    });

    // Check if the job has errors.
    if (isset($job->info()['status']['errorResult'])) {
      $error = $job->info()['status']['errorResult']['message'];
      $this->logger->alert('Error running job: ' . $error);

      return FALSE;
    }
    else {
      return TRUE;
    }
  }

  /**
   * Allows single inserts to BigQuery.
   *
   * @param string $dataset
   *   The name of the BigQuery dataset.
   * @param string $table
   *   The name of the BigQuery table.
   * @param array $response
   *   The response being submitted.
   * @param array $headerArray
   *   The header array.
   *
   * @todo Investigate enabling. Not being used because unsure of traffic
   */
  public function insertRow(string $dataset, string $table, array $response, array $headerArray) {
    $dataset = $this->bigQueryClient->dataset($dataset);
    $table = $dataset->table($table);

    foreach ($headerArray as $key => $label) {
      $output[$label] = isset($response[$key]) ? $response[$key] : '';
    }

    $insertResponse = $table->insertRow($output);

    if ($insertResponse->isSuccessful()) {
      print('Data streamed into BigQuery successfully' . PHP_EOL);
    }
    else {
      foreach ($insertResponse->failedRows() as $row) {
        foreach ($row['errors'] as $error) {
          printf('%s: %s' . PHP_EOL, $error['reason'], $error['message']);
        }
      }
    }
  }

}

==> simple_survey-8.x-1.16.tar.gz: simple_survey/src/Service/BigQueryConnect.php <==
<?php

namespace Drupal\simple_survey\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Datetime\DateFormatter;
use Google\Cloud\BigQuery\BigQueryClient;
use Google\Cloud\Core\ExponentialBackoff;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\simple_survey\Entity\SimpleSurvey;

/**
 * Drupal BigQuery Service.
 */
class BigQueryConnect implements ContainerInjectionInterface {

  /**
   * The module name.
   *
   * @var string
   */
  private $moduleName = 'simple_survey';

  /**
   * Is BigQuery Configured.
   *
   * @var bool
   */
  private $isConfigured = FALSE;

  /**
   * The big query client object.
   *
   * @var \Google\Cloud\BigQuery\BigQueryClient
   */
  private $bigQueryClient;

  /**
   * SimpleSurvey config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $simpleSurveyConfig;

  /**
   * Drupal config service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  private $configFactory;

  /**
   * Drupal logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Drupal messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The current survey.
   *
   * @var \Drupal\simple_survey\Entity\SimpleSurvey
   */
  public $survey;

  /**
   * Service config and DI.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Drupal Messenger service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config factory service.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   Date formatter service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   Drupal logger service.
   */
  public function __construct(
    MessengerInterface $messenger,
    ConfigFactoryInterface $configFactory,
    DateFormatter $dateFormatter,
    LoggerChannelFactoryInterface $loggerFactory
  ) {
    $this->messenger = $messenger;
    $this->configFactory = $configFactory;
    $this->dateFormatter = $dateFormatter;
    $this->logger = $loggerFactory->get($this->moduleName);
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('config.factory'),
      $container->get('date.formatter'),
      $container->get('logger.factory')
    );
  }

  /**
   * Sets the active survey.
   *
   * @param \Drupal\simple_survey\Entity\SimpleSurvey $survey
   *   The active survey.
   */
  public function setSurvey(SimpleSurvey $survey) {
    $this->survey = $survey;

    if ($this->survey->get('bigQueryKeyfile')) {
      $this->bigQueryClient = new BigQueryClient([
        'projectId' => $this->survey->get('bigQueryProjectId'),
        'keyFile' => json_decode($this->survey->get('bigQueryKeyfile'), TRUE),
      ]);

      $this->isConfigured = TRUE;
    }

  }

  /**
   * Checks Simple Survey configured to use BigQuery.
   *
   * @return bool
   *   Returns true if configured, false if not.
   */
  public function checkConfigured() {
    return $this->isConfigured;
  }

  /**
   * Send table data to BigQuery and overwrites existing data.
   *
   * @param string $dataset
   *   The name of the BigQuery dataset.
   * @param string $table
   *   The name of the BigQuery table.
   * @param mixed $csv
   *   The converted data.
   *
   * @return bool
   *   Returns true if updated, false if failed.
   */
  public function updateTable(string $dataset, string $table, $csv) {
    $dataset = $this->bigQueryClient->dataset($dataset);
    $table = $dataset->table($table);

    // Loads the CSV output.
    $loadConfig = $table->load($csv);
    // Sets the format to CSV.
    $loadConfig->sourceFormat('CSV');
    // Allows BigQuery to autodetect the header and create columns.
    $loadConfig->autodetect(TRUE);
    // Forces the BigQuery table to be overwritten.
    $loadConfig->writeDisposition('WRITE_TRUNCATE');

    $job = $table->runJob($loadConfig);

    // Poll the job until it is complete.
    $backoff = new ExponentialBackoff(10);
    $backoff->execute(function () use ($job) {
      $job->reload();
    });

    // Check if the job has errors.
    if (isset($job->info()['status']['errorResult'])) {
      $error = $job->info()['status']['errorResult']['message'];
      $this->logger->alert('Error running job: ' . $error);

      return FALSE;
    }
    else {
      return TRUE;
    }
  }

  /**
   * Allows single inserts to BigQuery.
   *
   * @param string $dataset
   *   The name of the BigQuery dataset.
   * @param string $table
   *   The name of the BigQuery table.
   * @param array $response
   *   The response being submitted.
   * @param array $headerArray
   *   The header array.
   *
   * @todo Investigate enabling. Not being used because unsure of traffic
   */
  public function insertRow(string $dataset, string $table, array $response, array $headerArray) {
    $dataset = $this->bigQueryClient->dataset($dataset);
    $table = $dataset->table($table);

    foreach ($headerArray as $key => $label) {
      $output[$label] = isset($response[$key]) ? $response[$key] : '';
    }

    $insertResponse = $table->insertRow($output);

    if ($insertResponse->isSuccessful()) {
      print('Data streamed into BigQuery successfully' . PHP_EOL);
    }
    else {
      foreach ($insertResponse->failedRows() as $row) {
        foreach ($row['errors'] as $error) {
          printf('%s: %s' . PHP_EOL, $error['reason'], $error['message']);
        }
      }
    }
  }

}
